@@ -1,4 +1,4 @@
-#include "image.h"
+#include "imager.h"
 
 static char *i_format_list[]={
 #ifdef HAVE_LIBJPEG
@@ -29,7 +29,6 @@ static char *i_format_list[]={
   "pnm",
   "bmp",
   "tga",
-  "rgb",
   "ifs",
   NULL};
 
