@@ -213,7 +213,7 @@ typedef struct {
 /* in memory tiff structure */
 typedef struct {
   /* the data we use as a tiff */
-  unsigned char *base;
+  const unsigned char *base;
   size_t size;
 
   /* intel or motorola byte order */
@@ -228,7 +228,7 @@ typedef struct {
   unsigned long next_ifd;
 } imtiff;
 
-static int tiff_init(imtiff *tiff, unsigned char *base, size_t length);
+static int tiff_init(imtiff *tiff, const unsigned char *base, size_t length);
 static int tiff_load_ifd(imtiff *tiff, unsigned long offset);
 static void tiff_final(imtiff *tiff);
 static void tiff_clear_ifd(imtiff *tiff);
@@ -286,7 +286,7 @@ Returns true if an EXIF header was seen.
 */
 
 int
-im_decode_exif(i_img *im, unsigned char *data, size_t length) {
+im_decode_exif(i_img *im, const unsigned char *data, size_t length) {
   imtiff tiff;
   unsigned long exif_ifd_offset = 0;
   unsigned long gps_ifd_offset = 0;
@@ -537,10 +537,10 @@ static tag_map light_source_values[] =
     { 9, "Fine weather" },
     { 10, "Cloudy weather" },
     { 11, "Shade" },
-    { 12, "Daylight fluorescent (D 5700  7100K)" },
-    { 13, "Day white fluorescent (N 4600  5400K)" },
-    { 14, "Cool white fluorescent (W 3900  4500K)" },
-    { 15, "White fluorescent (WW 3200  3700K)" },
+    { 12, "Daylight fluorescent (D 5700 \xD0 7100K)" },
+    { 13, "Day white fluorescent (N 4600 \xD0 5400K)" },
+    { 14, "Cool white fluorescent (W 3900 \xD0 4500K)" },
+    { 15, "White fluorescent (WW 3200 \xD0 3700K)" },
     { 17, "Standard light A" },
     { 18, "Standard light B" },
     { 19, "Standard light C" },
@@ -840,7 +840,7 @@ the first IFD (IFD0 in EXIF) in first_ifd_offset.
 */
 
 static int
-tiff_init(imtiff *tiff, unsigned char *data, size_t length) {
+tiff_init(imtiff *tiff, const unsigned char *data, size_t length) {
   int version;
 
   tiff->base = data;
