@@ -11,11 +11,6 @@
 #include <string.h>
 #include <errno.h>
 #include "imageri.h"
-#include <sys/stat.h>
-
-#ifdef IMAGER_POSIX_MMAP
-#include <sys/mman.h>
-#endif
 
 #define IOL_DEB(x)
 #define IOL_DEBs stderr
@@ -36,11 +31,6 @@ typedef struct io_blink {
 typedef struct {
   i_io_glue_t	base;
   int		fd;
-  off_t         size;
-  int size_set;
-#if defined(IMAGER_POSIX_MMAP)
-  void *mapped;
-#endif
 } io_fdseek;
 
 typedef struct {
@@ -109,7 +99,7 @@ iolayer.c implements the basic functions to create and destroy io_glue
 objects for Imager.  The typical usage pattern for data sources is:
 
    1. Create the source (io_new_fd)
-   2. Define how you want to get data from it
+   2. Define how you want to get data from it (io_reqmeth)
    3. read from it using the interface requested (ig->readdb, ig->mmapcb)
    4. Close the source, which 
       shouldn't really close the underlying source. (io_glue DESTROY)
@@ -124,16 +114,14 @@ Some of these functions are internal.
 */
 
 static void
-i_io_init(pIMCTX, io_glue *ig, int type, const i_io_glue_vtable_t *vtbl);
+i_io_init(pIMCTX, io_glue *ig, int type, i_io_readp_t readcb,
+	  i_io_writep_t writecb, i_io_seekp_t seekcb);
 
 static ssize_t fd_read(io_glue *ig, void *buf, size_t count);
 static ssize_t fd_write(io_glue *ig, const void *buf, size_t count);
 static off_t fd_seek(io_glue *ig, off_t offset, int whence);
 static int fd_close(io_glue *ig);
-static off_t fd_size(io_glue *ig);
-static int fd_mmap(io_glue *ig, const void **pdata, size_t *psize);
-static int fd_munmap(io_glue *ig);
-static void fd_destroy(io_glue *igo);
+static ssize_t fd_size(io_glue *ig);
 static const char *my_strerror(int err);
 static void i_io_setup_buffer(io_glue *ig);
 static void
@@ -151,9 +139,6 @@ static ssize_t buffer_read(io_glue *igo, void *buf, size_t count);
 static ssize_t buffer_write(io_glue *ig, const void *buf, size_t count);
 static int buffer_close(io_glue *ig);
 static off_t buffer_seek(io_glue *igo, off_t offset, int whence);
-static off_t buffer_size(io_glue *ig);
-static int buffer_mmap(io_glue *ig, const void **pdata, size_t *psize);
-static int buffer_munmap(io_glue *ig);
 static void buffer_destroy(io_glue *igo);
 static io_blink*io_blink_new(void);
 static void io_bchain_advance(io_ex_bchain *ieb);
@@ -164,54 +149,6 @@ static int bufchain_close(io_glue *ig);
 static off_t bufchain_seek(io_glue *ig, off_t offset, int whence);
 static void bufchain_destroy(io_glue *ig);
 
-static const i_io_glue_vtable_t
-bufchain_vtable =
-  {
-    bufchain_read,
-    bufchain_write,
-    bufchain_seek,
-    bufchain_close,
-    NULL,
-    bufchain_destroy
-  };
-
-static const i_io_glue_vtable_t
-buffer_vtable =
-  {
-    buffer_read,
-    buffer_write,
-    buffer_seek,
-    buffer_close,
-    buffer_size,
-    buffer_destroy,
-    buffer_mmap,
-    buffer_munmap
-  };
-
-static const i_io_glue_vtable_t
-fd_vtable =
-  {
-    fd_read,
-    fd_write,
-    fd_seek,
-    fd_close,
-    fd_size,
-    fd_destroy, /* destroy */
-    fd_mmap,
-    fd_munmap
-  };
-
-static const i_io_glue_vtable_t
-callback_vtable =
-  {
-    realseek_read,
-    realseek_write,
-    realseek_seek,
-    realseek_close,
-    NULL,
-    realseek_destroy
-  };
-
 /*
  * Methods for setting up data source
  */
@@ -239,7 +176,7 @@ im_io_new_bufchain(pIMCTX) {
 
   ig = mymalloc(sizeof(io_glue));
   memset(ig, 0, sizeof(*ig));
-  i_io_init(aIMCTX, ig, BUFCHAIN, &bufchain_vtable);
+  i_io_init(aIMCTX, ig, BUFCHAIN, bufchain_read, bufchain_write, bufchain_seek);
 
   ieb->offset = 0;
   ieb->length = 0;
@@ -252,6 +189,8 @@ im_io_new_bufchain(pIMCTX) {
   ieb->tail   = ieb->head;
   
   ig->exdata    = ieb;
+  ig->closecb   = bufchain_close;
+  ig->destroycb = bufchain_destroy;
 
   im_context_refinc(aIMCTX, "im_io_new_bufchain");
 
@@ -284,7 +223,7 @@ im_io_new_buffer(pIMCTX, const char *data, size_t len, i_io_closebufp_t closecb,
 
   ig = mymalloc(sizeof(io_buffer));
   memset(ig, 0, sizeof(*ig));
-  i_io_init(aIMCTX, &ig->base, BUFFER, &buffer_vtable);
+  i_io_init(aIMCTX, &ig->base, BUFFER, buffer_read, buffer_write, buffer_seek);
   ig->data      = data;
   ig->len       = len;
   ig->closecb   = closecb;
@@ -292,6 +231,9 @@ im_io_new_buffer(pIMCTX, const char *data, size_t len, i_io_closebufp_t closecb,
 
   ig->cpos   = 0;
   
+  ig->base.closecb   = buffer_close;
+  ig->base.destroycb = buffer_destroy;
+
   im_context_refinc(aIMCTX, "im_io_new_bufchain");
 
   return (io_glue *)ig;
@@ -324,9 +266,12 @@ im_io_new_fd(pIMCTX, int fd) {
 
   ig = mymalloc(sizeof(io_fdseek));
   memset(ig, 0, sizeof(*ig));
-  i_io_init(aIMCTX, &ig->base, FDSEEK, &fd_vtable);
+  i_io_init(aIMCTX, &ig->base, FDSEEK, fd_read, fd_write, fd_seek);
   ig->fd = fd;
 
+  ig->base.closecb   = fd_close;
+  ig->base.sizecb    = fd_size;
+  ig->base.destroycb = NULL;
   im_context_refinc(aIMCTX, "im_io_new_bufchain");
 
   im_log((aIMCTX, 1, "(%p) <- io_new_fd\n", ig));
@@ -389,9 +334,12 @@ im_io_new_cb(pIMCTX, void *p, i_io_readl_t readcb, i_io_writel_t writecb,
           "destroycb %p)\n", p, readcb, writecb, seekcb, closecb, destroycb));
   ig = mymalloc(sizeof(io_cb));
   memset(ig, 0, sizeof(*ig));
-  i_io_init(aIMCTX, &ig->base, CBSEEK, &callback_vtable);
+  i_io_init(aIMCTX, &ig->base, CBSEEK, realseek_read, realseek_write, realseek_seek);
   im_log((aIMCTX, 1, "(%p) <- io_new_cb\n", ig));
 
+  ig->base.closecb   = realseek_close;
+  ig->base.destroycb = realseek_destroy;
+
   ig->p         = p;
   ig->readcb    = readcb;
   ig->writecb   = writecb;
@@ -475,8 +423,8 @@ io_glue_destroy(io_glue *ig) {
   dIMCTXio(ig);
   im_log((aIMCTX, 1, "io_glue_DESTROY(ig %p)\n", ig));
 
-  if (ig->vtbl->destroycb)
-    ig->vtbl->destroycb(ig);
+  if (ig->destroycb)
+    ig->destroycb(ig);
 
   if (ig->buffer)
     myfree(ig->buffer);
@@ -1050,7 +998,7 @@ i_io_gets(io_glue *ig, char *buffer, size_t size, int eol) {
 }
 
 /*
-=item i_io_init(ig, type, vtbl)
+=item i_io_init(ig, readcb, writecb, seekcb)
 
 Do common initialization for io_glue objects.
 
@@ -1058,11 +1006,16 @@ Do common initialization for io_glue objects.
 */
 
 static void
-i_io_init(pIMCTX, io_glue *ig, int type, const i_io_glue_vtable_t *vtbl) {
+i_io_init(pIMCTX, io_glue *ig, int type, i_io_readp_t readcb, i_io_writep_t writecb,
+	  i_io_seekp_t seekcb) {
   ig->type = type;
   ig->exdata = NULL;
-
-  ig->vtbl = vtbl;
+  ig->readcb = readcb;
+  ig->writecb = writecb;
+  ig->seekcb = seekcb;
+  ig->closecb = NULL;
+  ig->sizecb = NULL;
+  ig->destroycb = NULL;
   ig->context = aIMCTX;
 
   ig->buffer = NULL;
@@ -1129,12 +1082,11 @@ i_io_dump(io_glue *ig, int flags) {
   fprintf(IOL_DEBs, "  type: %d\n", ig->type);  
   fprintf(IOL_DEBs, "  exdata: %p\n", ig->exdata);
   if (flags & I_IO_DUMP_CALLBACKS) {
-    fprintf(IOL_DEBs, "  vtable: %p\n", ig->vtbl);
-    fprintf(IOL_DEBs, "  readcb: %p\n", ig->vtbl->readcb);
-    fprintf(IOL_DEBs, "  writecb: %p\n", ig->vtbl->writecb);
-    fprintf(IOL_DEBs, "  seekcb: %p\n", ig->vtbl->seekcb);
-    fprintf(IOL_DEBs, "  closecb: %p\n", ig->vtbl->closecb);
-    fprintf(IOL_DEBs, "  sizecb: %p\n", ig->vtbl->sizecb);
+    fprintf(IOL_DEBs, "  readcb: %p\n", ig->readcb);
+    fprintf(IOL_DEBs, "  writecb: %p\n", ig->writecb);
+    fprintf(IOL_DEBs, "  seekcb: %p\n", ig->seekcb);
+    fprintf(IOL_DEBs, "  closecb: %p\n", ig->closecb);
+    fprintf(IOL_DEBs, "  sizecb: %p\n", ig->sizecb);
   }
   if (flags & I_IO_DUMP_BUFFER) {
     fprintf(IOL_DEBs, "  buffer: %p\n", ig->buffer);
@@ -1574,30 +1526,6 @@ buffer_seek(io_glue *igo, off_t offset, int whence) {
   /* FIXME: How about implementing this offset handling stuff? */
 }
 
-static off_t
-buffer_size(io_glue *igo) {
-  io_buffer *ig = (io_buffer *)igo;
-
-  return ig->len;
-}
-
-static int
-buffer_mmap(io_glue *igo, const void **pdata, size_t *psize) {
-  io_buffer *ig = (io_buffer *)igo;
-
-  /* already mapped */
-  *pdata = ig->data;
-  *psize = ig->len;
-
-  return 1;
-}
-
-static int
-buffer_munmap(io_glue *igo) {
-  /* nothing to do */
-  return 1;
-}
-
 static
 void
 buffer_destroy(io_glue *igo) {
@@ -2073,122 +2001,11 @@ static int fd_close(io_glue *ig) {
   return 0;
 }
 
-static off_t
-fd_size(io_glue *igo) {
-  dIMCTXio(igo);
-  io_fdseek *ig = (io_fdseek *)igo;
-
-  if (!ig->size_set) {
-    struct stat st;
-    if (fstat(ig->fd, &st) == 0) {
-      ig->size = st.st_size;
-    }
-    else {
-      ig->size = -1;
-    }
-    ig->size_set = 1;
-  }
+static ssize_t fd_size(io_glue *ig) {
+  dIMCTXio(ig);
+  im_log((aIMCTX, 1, "fd_size(ig %p) unimplemented\n", ig));
   
-  return ig->size;
-}
-
-static int
-fd_mmap(io_glue *igo, const void **pdata, size_t *psize) {
-#if defined(IMAGER_POSIX_MMAP)
-  dIMCTXio(igo);
-  io_fdseek *ig = (io_fdseek *)igo;
-  void *data;
-  if (!ig->size_set)
-    fd_size(igo);
-  if (ig->size <= 0)
-    return 0;
-  if (ig->size > aIMCTX->max_mmap_size) {
-    im_push_error(aIMCTX, 0, "file larger than max mmap size");
-    return 0;
-  }
-  data = mmap(NULL, ig->size, PROT_READ, MAP_SHARED, ig->fd, 0);
-  if (!data) {
-    return 0;
-  }
-  
-  ig->mapped = data;
-  *pdata = data;
-  *psize = ig->size;
-
-  return 1;
-#else
-  return 0;
-#endif
-}
-
-static int
-fd_munmap(io_glue *igo) {
-#if defined(IMAGER_POSIX_MMAP)
-  io_fdseek *ig = (io_fdseek *)igo;
-
-  if (ig->mapped) {
-    munmap(ig->mapped, ig->size);
-    ig->mapped = NULL;
-    return 1;
-  }
-  return 0;
-#else
-  return 0;
-#endif
-}
-
-static void
-fd_destroy(io_glue *igo) {
-#if defined(IMAGER_POSIX_MMAP)
-  io_fdseek *ig = (io_fdseek *)igo;
-
-  if (ig->mapped) {
-    fd_munmap(igo);
-  }
-#endif
-  
-}
-
-/*
-=item im_io_get_max_mmap_size()
-
-  size_t size = im_io_get_max_mmap_size(aIMCTX);
-  size_t size = i_io_get_max_mmap_size();
-
-Fetch the maximum address space than can be mapped by i_io_mmap() for
-files.
-
-=cut
-*/
-
-size_t
-im_io_get_max_mmap_size(pIMCTX) {
-  return aIMCTX->max_mmap_size;
-}
-
-/*
-=item im_io_set_max_mmap_size()
-
-  int ok size = im_io_set_max_mmap_size(aIMCTX, new_size);
-  int ok size = i_io_set_max_mmap_size(new_size);
-
-Set the maximum address space than can be mapped by i_io_mmap() for
-files.
-
-=cut
-*/
-
-int
-im_io_set_max_mmap_size(pIMCTX, size_t new_size) {
-  im_clear_error(aIMCTX);
-
-  if (new_size > im_size_t_max / 2) {
-    im_push_error(aIMCTX, 0, "set_max_map_size: new size too large (> half address space)");
-    return 0;
-  }
-  aIMCTX->max_mmap_size = new_size;
-
-  return 1;
+  return -1;
 }
 
 
@@ -2205,103 +2022,3 @@ Imager(3)
 
 =cut
 */
-
-/*
-
-Document the I/O macros.
-
-=item i_io_type()
-=category I/O Layers
-
-Return the type of an I/O layer object.
-
-  int type = i_io_type(io);
-
-=cut
-=item i_io_raw_read()
-=category I/O Layers
-
-Raw read from an I/O layer object.  This does not mix well with the
-buffer I/O functions.
-
-  char buf[SIZE];
-  ssize_t rd_size = i_io_raw_read(io, buf, SIZE);
-
-=cut
-=item i_io_raw_write()
-=category I/O Layers
-
-Raw write to an I/O layer object.  This does not mix well with the
-buffer I/O functions.
-
-  char data[SIZE] = "...";
-  ssize_t wr_size = i_io_raw_write(io, buf, SIZE);
-
-=cut
-=item i_io_raw_seek()
-=category I/O Layers
-
-Raw seek in an I/O layer object.  This does not mix well with the
-buffer I/O functions.
-
-  off_t pos = i_io_raw_seek(io, offset, seek_type);
-
-=cut
-=item i_io_raw_close()
-=category I/O Layers
-
-Raw close an I/O layer object.  This will not flush any buffered data.
-
-  int result = i_io_raw_close(io)
-
-=cut
-=item i_io_is_buffered()
-=category I/O Layers
-
-Returns true if the I/O layer object is buffered.
-
-=cut
-=item i_io_mmap()
-=category I/O Layers
-
-If the I/O object allows it, return the address and size of the data.
-For C<fd> objects this will attempt to mmap() the file into memory.
-
-The memory may not be written to.
-
-  const void *p;
-  size_t size;
-  if (i_io_mmap(io, &p, &size)) {
-    i_io_munmap(p);
-  }
-
-=cut
-=item i_io_munmap()
-=category I/O Layers
-
-Remove the memory mapping created by i_io_mmap().
-
-  i_io_munmap(io);
-
-=cut
-=item i_io_nextc()
-=category I/O Layers
-
-Currently broken.
-
-=cut
-=item i_io_eof()
-=category I/O Layers
-
-Return true if the stream is at end of file.
-
-=cut
-=item i_io_error()
-=category I/O Layers
-
-Return true if the stream is in an error state.  i_io_seek() will
-clear the error state.
-
-=cut
-
-*/
