@@ -29,9 +29,8 @@ typedef ssize_t(*i_io_readp_t) (io_glue *ig, void *buf, size_t count);
 typedef ssize_t(*i_io_writep_t)(io_glue *ig, const void *buf, size_t count);
 typedef off_t  (*i_io_seekp_t) (io_glue *ig, off_t offset, int whence);
 typedef int    (*i_io_closep_t)(io_glue *ig);
-typedef off_t  (*i_io_sizep_t) (io_glue *ig);
-typedef int    (*i_io_mmapp_t)(io_glue *ig, const void **pdata, size_t *psize);
-typedef int    (*i_io_munmapp_t)(io_glue *ig);
+typedef ssize_t(*i_io_sizep_t) (io_glue *ig);
+
 typedef void   (*i_io_closebufp_t)(void *p);
 typedef void (*i_io_destroyp_t)(i_io_glue_t *ig);
 
@@ -43,7 +42,7 @@ typedef ssize_t(*i_io_writel_t)(void *p, const void *buf, size_t count);
 typedef off_t  (*i_io_seekl_t) (void *p, off_t offset, int whence);
 typedef int    (*i_io_closel_t)(void *p);
 typedef void   (*i_io_destroyl_t)(void *p);
-typedef off_t  (*i_io_sizel_t) (void *p);
+typedef ssize_t(*i_io_sizel_t) (void *p);
 
 extern char *io_type_names[];
 
@@ -51,21 +50,15 @@ extern char *io_type_names[];
 
 /* Structures to describe data sources */
 
-typedef struct {
+struct i_io_glue_t {
+  io_type type;
+  void *exdata;
   i_io_readp_t	readcb;
   i_io_writep_t	writecb;
   i_io_seekp_t	seekcb;
   i_io_closep_t	closecb;
   i_io_sizep_t	sizecb;
   i_io_destroyp_t destroycb;
-  i_io_mmapp_t mmapcb;
-  i_io_munmapp_t munmapcb;
-} i_io_glue_vtable_t;
-
-struct i_io_glue_t {
-  const i_io_glue_vtable_t *vtbl;
-  io_type type;
-  void *exdata;
   unsigned char *buffer;
   unsigned char *read_ptr;
   unsigned char *read_end;
@@ -91,16 +84,11 @@ struct i_io_glue_t {
 #define I_IO_DUMP_DEFAULT (I_IO_DUMP_BUFFER | I_IO_DUMP_STATUS)
 
 #define i_io_type(ig) ((ig)->source.ig_type)
-#define i_io_raw_read(ig, buf, size) ((ig)->vtbl->readcb((ig), (buf), (size)))
-#define i_io_raw_write(ig, data, size) ((ig)->vtbl->writecb((ig), (data), (size)))
-#define i_io_raw_seek(ig, offset, whence) ((ig)->vtbl->seekcb((ig), (offset), (whence)))
-#define i_io_raw_close(ig) ((ig)->vtbl->closecb(ig))
+#define i_io_raw_read(ig, buf, size) ((ig)->readcb((ig), (buf), (size)))
+#define i_io_raw_write(ig, data, size) ((ig)->writecb((ig), (data), (size)))
+#define i_io_raw_seek(ig, offset, whence) ((ig)->seekcb((ig), (offset), (whence)))
+#define i_io_raw_close(ig) ((ig)->closecb(ig))
 #define i_io_is_buffered(ig) ((int)((ig)->buffered))
-#define i_io_size(ig) ((ig)->vtbl->sizecb ? (ig)->vtbl->sizecb(ig) : (off_t)-1)
-#define i_io_mmap(ig, pdata, psize) \
-  ((ig)->vtbl->mmapcb ? (ig)->vtbl->mmapcb((ig), (pdata), (psize)) : 0)
-#define i_io_munmap(ig) \
-  ((ig)->vtbl->munmapcb ? (ig)->vtbl->munmapcb(ig) : 0)
 
 #define i_io_getc(ig) \
   ((ig)->read_ptr < (ig)->read_end ? \
