@@ -13,17 +13,19 @@ struct i_mutex_tag {
 };
 
 i_mutex_t
-i_mutex_create(void) {
+i_mutex_new(void) {
   i_mutex_t m;
 
-  m = mymalloc(sizeof(*m));
+  m = malloc(sizeof(*m));
+  if (!m)
+    i_fatal(3, "Cannot allocate mutex object");
 
   return m;
 }
 
 void
 i_mutex_destroy(i_mutex_t m) {
-  myfree(m);
+  free(m);
 }
 
 void
