@@ -22,13 +22,12 @@ Basic 8-bit/sample paletted image
 
 #include "imager.h"
 #include "imageri.h"
-#include "imapiver.h"
-#include <assert.h>
-
-/* no longer in the vtable, but still used */
-static int i_ppix_p(i_img *im, i_img_dim x, i_img_dim y, const i_color *val);
 
 #define PALEXT(im) ((i_img_pal_ext*)((im)->ext_data))
+static int i_ppix_p(i_img *im, i_img_dim x, i_img_dim y, const i_color *val);
+static int i_gpix_p(i_img *im, i_img_dim x, i_img_dim y, i_color *val);
+static i_img_dim i_glin_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals);
+static i_img_dim i_plin_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals);
 static i_img_dim i_gsamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samps, int const *chans, int chan_count);
 static i_img_dim i_gpal_p(i_img *pm, i_img_dim l, i_img_dim r, i_img_dim y, i_palidx *vals);
 static i_img_dim i_ppal_p(i_img *pm, i_img_dim l, i_img_dim r, i_img_dim y, const i_palidx *vals);
@@ -45,34 +44,29 @@ i_psamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_sample_t *sa
 static i_img_dim 
 i_psampf_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fsample_t *samps, const int *chans, int chan_count);
 
-static i_image_data_alloc_t *
-i_data_pal(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
-           void **pdata, size_t *psize, int *pextra);
+static i_img IIM_base_8bit_pal =
+{
+  0, /* channels set */
+  0, 0, 0, /* xsize, ysize, bytes */
+  ~0U, /* ch_mask */
+  i_8_bits, /* bits */
+  i_palette_type, /* type */
+  0, /* virtual */
+  NULL, /* idata */
+  { 0, 0, NULL }, /* tags */
+  NULL, /* ext_data */
 
-static const i_img_vtable
-vtable_pal = {
-  IMAGER_API_LEVEL,
-  
+  i_ppix_p, /* i_f_ppix */
+  i_ppixf_fp, /* i_f_ppixf */
+  i_plin_p, /* i_f_plin */
+  i_plinf_fp, /* i_f_plinf */
+  i_gpix_p, /* i_f_gpix */
+  i_gpixf_fp, /* i_f_gpixf */
+  i_glin_p, /* i_f_glin */
+  i_glinf_fp, /* i_f_glinf */
   i_gsamp_p, /* i_f_gsamp */
   i_gsampf_fp, /* i_f_gsampf */
 
-  i_psamp_p,
-  i_psampf_p,
-
-  i_destroy_p, /* i_f_destroy */
-
-  i_gsamp_bits_fb,
-  i_psamp_bits_fb, /* i_f_psamp_bits */
-
-  i_gslin_fallback,
-  i_gslinf_fallback,
-  i_pslin_fallback,
-  i_pslinf_fallback,
-  
-  i_data_pal,
-
-  NULL, /* i_f_imageop */
-
   i_gpal_p, /* i_f_gpal */
   i_ppal_p, /* i_f_ppal */
   i_addcolors_p, /* i_f_addcolors */
@@ -80,7 +74,15 @@ vtable_pal = {
   i_colorcount_p, /* i_f_colorcount */
   i_maxcolors_p, /* i_f_maxcolors */
   i_findcolor_p, /* i_f_findcolor */
-  i_setcolors_p /* i_f_setcolors */
+  i_setcolors_p, /* i_f_setcolors */
+
+  i_destroy_p, /* i_f_destroy */
+
+  i_gsamp_bits_fb,
+  NULL, /* i_f_psamp_bits */
+  
+  i_psamp_p,
+  i_psampf_p
 };
 
 /*
@@ -134,21 +136,21 @@ im_img_pal_new(pIMCTX, i_img_dim x, i_img_dim y, int channels, int maxpal) {
     return NULL;
   }
 
-  im = im_img_new(aIMCTX, &vtable_pal, x, y, channels, 0, i_8_bits);
-  if (im == NULL)
-    return NULL;
-
-  im->type = i_palette_type;
-
+  im = i_img_alloc();
+  memcpy(im, &IIM_base_8bit_pal, sizeof(i_img));
   palext = mymalloc(sizeof(i_img_pal_ext));
   palext->pal = mymalloc(sizeof(i_color) * maxpal);
   palext->count = 0;
   palext->alloc = maxpal;
   palext->last_found = -1;
   im->ext_data = palext;
-
+  i_tags_new(&im->tags);
   im->bytes = bytes;
-  im->idata = myzmalloc(im->bytes);
+  im->idata = mymalloc(im->bytes);
+  im->channels = channels;
+  memset(im->idata, 0, im->bytes);
+  im->xsize = x;
+  im->ysize = y;
 
   i_img_init(im);
   
@@ -167,20 +169,12 @@ same width, height and channels.
 
 =cut
 */
-
-static void
-i_img_rgb_convert(i_img *targ, i_img *src) {
-  int totalch = i_img_totalchannels(targ);
-  i_sample_t *row = mymalloc(sizeof(i_color) * targ->xsize);
+static void i_img_rgb_convert(i_img *targ, i_img *src) {
+  i_color *row = mymalloc(sizeof(i_color) * targ->xsize);
   i_img_dim y;
-
-  assert(i_img_channels(targ) == i_img_channels(src));
-  assert(i_img_extrachannels(targ) == i_img_extrachannels(src));
-  assert(i_img_bits(targ) == i_8_bits);
-
   for (y = 0; y < targ->ysize; ++y) {
-    i_gsamp(src, 0, src->xsize, y, row, NULL, totalch);
-    i_psamp(targ, 0, src->xsize, y, row, NULL, totalch);
+    i_glin(src, 0, src->xsize, y, row);
+    i_plin(targ, 0, src->xsize, y, row);
   }
   myfree(row);
 }
@@ -198,9 +192,8 @@ The conversion cannot be done for virtual images.
 */
 int
 i_img_to_rgb_inplace(i_img *im) {
-  i_img *temp;
+  i_img temp;
   dIMCTXim(im);
-  i_img sw;
 
   if (i_img_virtual(im))
     return 0;
@@ -208,14 +201,15 @@ i_img_to_rgb_inplace(i_img *im) {
   if (im->type == i_direct_type)
     return 1; /* trivial success */
 
-  temp = i_img_8_new(im->xsize, im->ysize, im->channels);
-  i_img_rgb_convert(temp, im);
+  i_img_empty_ch(&temp, im->xsize, im->ysize, im->channels);
+  i_img_rgb_convert(&temp, im);
 
   /* nasty hack */
-  sw = *im;
-  *im = *temp;
-  *temp = sw;
-  i_img_destroy(temp);
+  i_img_exorcise(im);
+  *im = temp;
+
+  /* i_img_empty_ch() calls i_img_init() which takes a ref */
+  im_context_refdec(aIMCTX, "img_destroy");
 
   return 1;
 }
@@ -233,11 +227,6 @@ i_img *i_img_to_pal(i_img *src, i_quantize *quant) {
   dIMCTXim(src);
 
   i_clear_error();
-
-  if (src->channels == 0) {
-    i_push_error(0, "no color channels found to build a palette");
-    return NULL;
-  }
   
   i_quant_makemap(quant, &src, 1);
   result = i_quant_translate(quant, src);
@@ -263,21 +252,12 @@ i_img *i_img_to_pal(i_img *src, i_quantize *quant) {
 /*
 =item i_img_to_rgb(i_img *src)
 
-Create an 8-bit/sample version of the source image.
-
 =cut
 */
-
 i_img *
 i_img_to_rgb(i_img *src) {
   dIMCTXim(src);
-  i_img *im =
-    im_img_8_new_extra(aIMCTX, i_img_get_width(src), i_img_get_height(src), i_img_channels(src),
-                       i_img_extrachannels(src));
-
-  if (!im)
-    return NULL;
-
+  i_img *im = i_img_empty_ch(NULL, src->xsize, src->ysize, src->channels);
   i_img_rgb_convert(im, src);
 
   return im;
@@ -349,6 +329,92 @@ i_ppix_p(i_img *im, i_img_dim x, i_img_dim y, const i_color *val) {
   }
 }
 
+/*
+=item i_gpix_p(i_img *im, i_img_dim x, i_img_dim y, i_color *val)
+
+Retrieve a pixel, converting from a palette index to a color.
+
+=cut
+*/
+static int i_gpix_p(i_img *im, i_img_dim x, i_img_dim y, i_color *val) {
+  i_palidx which;
+  if (x < 0 || x >= im->xsize || y < 0 || y >= im->ysize) {
+    return -1;
+  }
+  which = ((i_palidx *)im->idata)[x + y * im->xsize];
+  if (which > PALEXT(im)->count)
+    return -1;
+  *val = PALEXT(im)->pal[which];
+
+  return 0;
+}
+
+/*
+=item i_glinp(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals)
+
+Retrieve a row of pixels.
+
+=cut
+*/
+static i_img_dim i_glin_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals) {
+  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
+    int palsize = PALEXT(im)->count;
+    i_color *pal = PALEXT(im)->pal;
+    i_palidx *data;
+    i_img_dim count, i;
+    if (r > im->xsize)
+      r = im->xsize;
+    data = ((i_palidx *)im->idata) + l + y * im->xsize;
+    count = r - l;
+    for (i = 0; i < count; ++i) {
+      i_palidx which = *data++;
+      if (which < palsize)
+        vals[i] = pal[which];
+    }
+    return count;
+  }
+  else {
+    return 0;
+  }
+}
+
+/*
+=item i_plin_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals)
+
+Write a line of color data to the image.
+
+If any color value is not in the image when the image is converted to 
+RGB.
+
+=cut
+*/
+static i_img_dim 
+i_plin_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals) {
+  i_img_dim count, i;
+  i_palidx *data;
+  i_palidx which;
+  if (y >=0 && y < im->ysize && l < im->xsize && l >= 0) {
+    if (r > im->xsize)
+      r = im->xsize;
+    data = ((i_palidx *)im->idata) + l + y * im->xsize;
+    count = r - l;
+    for (i = 0; i < count; ++i) {
+      if (i_findcolor(im, vals+i, &which)) {
+        ((i_palidx *)data)[i] = which;
+      }
+      else {
+        if (i_img_to_rgb_inplace(im)) {
+          return i+i_plin(im, l+i, r, y, vals+i);
+        }
+      }
+    }
+    return count;
+  }
+  else {
+    return 0;
+  }
+}
+
 /*
 =item i_gsamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samps, int chans, int chan_count)
 
@@ -356,8 +422,6 @@ i_ppix_p(i_img *im, i_img_dim x, i_img_dim y, const i_color *val) {
 */
 static i_img_dim i_gsamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samps, 
               int const *chans, int chan_count) {
-  i_assert_valid_channels(im, chans, chan_count);
-
   int ch;
   if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
     int palsize = PALEXT(im)->count;
@@ -370,6 +434,13 @@ static i_img_dim i_gsamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_s
     count = 0;
     w = r - l;
     if (chans) {
+      for (ch = 0; ch < chan_count; ++ch) {
+        if (chans[ch] < 0 || chans[ch] >= im->channels) {
+	  dIMCTXim(im);
+          im_push_errorf(aIMCTX, 0, "No channel %d in this image", chans[ch]);
+        }
+      }
+
       for (i = 0; i < w; ++i) {
         i_palidx which = *data++;
         if (which < palsize) {
@@ -381,6 +452,12 @@ static i_img_dim i_gsamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_s
       }
     }
     else {
+      if (chan_count <= 0 || chan_count > im->channels) {
+	dIMCTXim(im);
+	im_push_errorf(aIMCTX, 0, "chan_count %d out of range, must be >0, <= channels", 
+		      chan_count);
+	return 0;
+      }
       for (i = 0; i < w; ++i) {
         i_palidx which = *data++;
         if (which < palsize) {
@@ -572,8 +649,6 @@ handlers.
 static i_img_dim 
 i_psamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 	  const i_sample_t *samps, const int *chans, int chan_count) {
-  i_assert_valid_channels(im, chans, chan_count);
-
   if (y >=0 && y < im->ysize && l < im->xsize && l >= 0) {
     i_img_dim count = 0;
     int ch;
@@ -582,33 +657,44 @@ i_psamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
       r = im->xsize;
       
     if (chans) {
-      while (l < r && im->type == i_palette_type) {
+      /* make sure we have good channel numbers */
+      for (ch = 0; ch < chan_count; ++ch) {
+        if (chans[ch] < 0 || chans[ch] >= im->channels) {
+	  dIMCTXim(im);
+          im_push_errorf(aIMCTX, 0, "No channel %d in this image", chans[ch]);
+          return -1;
+        }
+      }
+      while (l < r) {
 	i_color c;
 	
 	i_gpix(im, l, y, &c);
 	for (ch = 0; ch < chan_count; ++ch)
 	  c.channel[chans[ch]] = *samps++;
-        i_ppix_p(im, l, y, &c);
+	i_ppix(im, l, y, &c);
 	count += chan_count;
 	++l;
       }
     }
     else {
-      while (l < r && im->type == i_palette_type) {
+      if (chan_count <= 0 || chan_count > im->channels) {
+	dIMCTXim(im);
+	im_push_errorf(aIMCTX, 0, "chan_count %d out of range, must be >0, <= channels", 
+		      chan_count);
+	return -1;
+      }
+
+      while (l < r) {
 	i_color c;
 	
 	i_gpix(im, l, y, &c);
 	for (ch = 0; ch < chan_count; ++ch)
 	  c.channel[ch] = *samps++;
-        i_ppix_p(im, l, y, &c);
+	i_ppix(im, l, y, &c);
 	count += chan_count;
 	++l;
       }
     }
-    if (l < r) {
-      /* at this point it's no longer a paletted image */
-      count += i_psamp(im, l, r, y, samps, chans, chan_count);
-    }
 
     return count;
   }
@@ -619,43 +705,6 @@ i_psamp_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
   }
 }
 
-/*
-=item i_ppixf_fp()
-
-=cut
-*/
-
-IMAGER_STATIC_INLINE int
-i_ppixf_fp(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *pix) {
-  i_color temp;
-  int ch;
-
-  for (ch = 0; ch < im->channels; ++ch)
-    temp.channel[ch] = SampleFTo8(pix->channel[ch]);
-  
-  return i_ppix(im, x, y, &temp);
-}
-
-/*
-=item i_gpixf_fp(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix)
-
-=cut
-*/
-
-IMAGER_STATIC_INLINE int
-i_gpixf_fp(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix) {
-  i_color temp;
-  int ch;
-
-  if (i_gpix(im, x, y, &temp) == 0) {
-    for (ch = 0; ch < im->channels; ++ch)
-      pix->channel[ch] = Sample8ToF(temp.channel[ch]);
-    return 0;
-  }
-  else 
-    return -1;
-}
-
 /*
 =item i_psampf_p(im, l, r, y, samps, chans, chan_count)
 
@@ -696,7 +745,7 @@ i_psampf_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 	i_gpixf(im, l, y, &c);
 	for (ch = 0; ch < chan_count; ++ch)
 	  c.channel[chans[ch]] = *samps++;
-	i_ppixf_fp(im, l, y, &c);
+	i_ppixf(im, l, y, &c);
 	count += chan_count;
 	++l;
       }
@@ -715,7 +764,7 @@ i_psampf_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 	i_gpixf(im, l, y, &c);
 	for (ch = 0; ch < chan_count; ++ch)
 	  c.channel[ch] = *samps++;
-	i_ppixf_fp(im, l, y, &c);
+	i_ppixf(im, l, y, &c);
 	count += chan_count;
 	++l;
       }
@@ -730,22 +779,6 @@ i_psampf_p(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
   }
 }
 
-static i_image_data_alloc_t *
-i_data_pal(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
-           void **pdata, size_t *psize, int *pextra) {
-  if (bits != i_8_bits
-      || layout != idl_palette
-      || layout != im->channels) {
-    return i_img_data_fallback(im, layout, bits, flags, pdata, psize, pextra);
-  }
-
-  *pdata = im->idata;
-  *psize = im->bytes;
-  *pextra = 0;
-
-  return i_new_image_data_alloc_def(im);
-}
-
 /*
 =back
 
