@@ -27,6 +27,8 @@ MainLoop;
 
 =head1 NAME
 
+=for stopwords tk-photo.pl
+
 tk-photo.pl - display an Imager image under Tk
 
 =head1 SYNOPSIS
@@ -58,10 +60,10 @@ the C<-data> parameter.
 
 =head1 REVISION
 
-$Revision: 830 $
+$Revision$
 
 =head1 AUTHOR
 
-Tony Cook <tony@imager.perl.org>
+Tony Cook <tonyc@cpan.org>
 
 =cut
